@@ -20,6 +20,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <libgnome/gnome-help.h>
 #include <gtk/gtk.h>
+#include <gconf/gconf-client.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-control.h>
@@ -39,7 +40,9 @@ extern void destroy_cache_hashTable (void);
 static GNOME_Media_CDDBTrackEditor track_editor = CORBA_OBJECT_NIL;
 
 static gboolean position_auto_update=TRUE;
+static gboolean stopped_state=FALSE;
 static gboolean position_update_ignore_event=FALSE;
+static gboolean remaining_time_mode=FALSE;
 static gchar* discid_prev;
 
 static void
@@ -183,11 +186,16 @@ play_cb (GtkButton *button,
 			end_track = -1;
 			endmsf = NULL;
 		} else {
-			end_track = status->track + 1;
-			msf.minute = 0;
-			msf.second = 0;
-			msf.frame = 0;
-			endmsf = &msf;
+			if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
+				end_track = -1;
+				endmsf = NULL;	
+			} else {	
+				end_track = status->track + 1;
+				msf.minute = 0;
+				msf.second = 0;
+				msf.frame = 0;
+				endmsf = &msf;
+			}
 		}
 		if (gnome_cdrom_play (gcd->cdrom, status->track,
 				      &status->relative, end_track, endmsf, &error) == FALSE) {
@@ -208,6 +216,7 @@ play_cb (GtkButton *button,
 		break;
 
 	case GNOME_CDROM_AUDIO_COMPLETE:
+		stopped_state = FALSE;
 	case GNOME_CDROM_AUDIO_STOP:
 		msf.minute = 0;
 		msf.second = 0;
@@ -265,6 +274,8 @@ stop_cb (GtkButton *button,
 		return;
 	}
 
+	stopped_state = TRUE;
+
 	if (gcd->current_image == gcd->pause_image) {
 		AtkObject *aob;
 
@@ -489,6 +500,8 @@ status_ok (GnomeCD *gcd,
 	int track;
 	char *text;
 	char *current_time;
+
+	gboolean remaining_time_temp = gconf_client_get_bool (gcd->client, "/apps/gnome-cd/show-remaining-time",NULL);
 	
 	/* Allow the track editor to work */
 	gtk_widget_set_sensitive (gcd->trackeditor_b, TRUE);
@@ -529,10 +542,24 @@ status_ok (GnomeCD *gcd,
 		}
 		
 /*  		cd_display_clear (CD_DISPLAY (gcd->display)); */
-		text = g_strdup_printf ("%d:%02d / %d:%02d", 
+		if (remaining_time_temp) {
+			gint min = status->length.minute - status->relative.minute;
+			gint sec = status->length.second - status->relative.second;
+			if(sec < 0) {
+				min--;
+				sec = 60 - (-1 * sec);
+			}
+			text = g_strdup_printf ("- %d:%02d", min, sec);
+			current_time = g_strdup_printf ("Remaining time: %s", text);
+        } else {
+           	text = g_strdup_printf ("%d:%02d / %d:%02d",
 					status->relative.minute, status->relative.second,
 					status->length.minute, status->length.second);
-		current_time = g_strdup_printf ("%d:%02d",status->relative.minute, status->relative.second);
+	 	      		current_time = g_strdup_printf ("Current time: %s", text);
+        }
+
+		/* Displaying tooltip */
+		gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET(gcd->display), current_time, NULL);
 		
 		/* update position slider */
 		if(position_auto_update && (status->length.minute!=0 ||
@@ -558,12 +585,12 @@ status_ok (GnomeCD *gcd,
 			
 		/* Update the tray icon tooltip */
 		if (gcd->disc_info != NULL) {
-			text = g_strdup_printf (_("Playing %s - %s\nCurrent Time: %s"),
+			text = g_strdup_printf (_("Playing %s - %s\n%s"),
 						gcd->disc_info->artist ? gcd->disc_info->artist : _("Unknown Artist"),
 						gcd->disc_info->title ? gcd->disc_info->title : _("Unknown Album"),
 						current_time);
 		} else {
-			text = g_strdup_printf (_("Playing\nCurrent Time: %s"), current_time);
+			text = g_strdup_printf (_("Playing\n: %s"), current_time);
 		}
 		if (gcd->tray_tips)	
 			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, text, NULL);
@@ -591,26 +618,31 @@ status_ok (GnomeCD *gcd,
 		break;
 		
 	case GNOME_CDROM_AUDIO_COMPLETE:
-		if (gcd->cdrom->loopmode == GNOME_CDROM_LOOP) {
+		if (gcd->cdrom->loopmode == GNOME_CDROM_LOOP && stopped_state == FALSE) {
 			if (gcd->cdrom->playmode == GNOME_CDROM_WHOLE_CD) {
 				/* Around we go */
 				play_cb (NULL, gcd);
 			} else {
-				GnomeCDRomMSF msf;
+				GnomeCDRomMSF msf, *endmsf;
 				int start_track, end_track;
 				GError *error = NULL;
 				
-				/* CD has gone to the start of the next track.
-				   Track we want to loop is track - 1 */
-				start_track = status->track - 1;
-				end_track = status->track;
-				
+				/* CD is still in the same track */
 				msf.minute = 0;
 				msf.second = 0;
 				msf.frame = 0;
 				
+				start_track = status->track;
+				if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
+					end_track = -1;
+					endmsf = NULL;
+				} else {
+					end_track = status->track + 1;
+					endmsf = &msf;
+				}
+				
 				if (gnome_cdrom_play (gcd->cdrom, start_track, &msf,
-						      end_track, &msf, &error) == FALSE) {
+						      end_track, endmsf, &error) == FALSE) {
 					gcd_warning ("%s", error);
 					g_error_free (error);
 					
@@ -634,6 +666,8 @@ status_ok (GnomeCD *gcd,
 			/* Update tray icon tooltip */
 			if (gcd->tray_tips)	
 				gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("CD Player"), NULL);
+			if (gcd->tooltips)
+				gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET(gcd->display), _("CD Player"), NULL);
 
 			/* Initialise the display time to 0:00 in stopped state*/
 			cd_display_set_line (CD_DISPLAY (gcd->display),
@@ -749,6 +783,7 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 
         /* everything below are states where we're not playing */
 	case GNOME_CDROM_STATUS_NO_DISC:
+	case GNOME_CDROM_STATUS_EMPTY_DISC:
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
 			gcd->disc_info = NULL;
@@ -771,7 +806,10 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		gtk_widget_set_sensitive (gcd->eject_b, TRUE);
 
 		cd_display_clear (CD_DISPLAY (gcd->display));
-		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("No disc"));
+		if (status->cd == GNOME_CDROM_STATUS_EMPTY_DISC)
+			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("Empty disc"));
+		else
+			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("No disc")); 
 		gnome_cd_set_window_title (gcd, NULL, NULL);
 		break;
 
@@ -927,6 +965,15 @@ loopmode_changed_cb (GtkWidget *display,
 	gcd->cdrom->loopmode = mode;
 }
 
+void
+remainingtime_mode_changed_cb (GtkWidget *display,
+                    GnomeCDRomMode mode,
+                    GnomeCD *gcd)
+{
+	remaining_time_mode = !remaining_time_mode;
+	gconf_client_set_bool (gcd->client, "/apps/gnome-cd/show-remaining-time", remaining_time_mode, NULL);
+}
+
 void
 playmode_changed_cb (GtkWidget *display,
 		     GnomeCDRomMode mode,
@@ -1187,7 +1234,7 @@ volume_changed (GtkRange *range,
 	scaled_volume = (volume/255)*100;
 	volume_level = g_strdup_printf (_("Volume %d%%"), scaled_volume);
 	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->slider), volume_level, NULL);
-	gconf_client_set_int (client, "/apps/gnome-cd/volume", scaled_volume, NULL);
+	gconf_client_set_int (gcd->client, "/apps/gnome-cd/volume", scaled_volume, NULL);
 	g_free (volume_level);
 }
 
@@ -1230,8 +1277,13 @@ position_changed (GtkRange *range,
 	msf2.frame = 0;
 	
 	if (gcd->cdrom->playmode == GNOME_CDROM_SINGLE_TRACK) {
-		end_track = status->track + 1;
-		endmsf = &msf2;
+		if (gcd->disc_info && status->track >= gcd->disc_info->ntracks) {
+			end_track = -1;
+			endmsf = NULL;
+		} else {	
+			end_track = status->track + 1;
+			endmsf = &msf2;
+		}
 	} else {
 		end_track = -1;
 		endmsf = NULL;
