@@ -347,7 +347,7 @@ window_destroy_cb (GtkWidget *window,
 	destroy_track_editor ();
 	
 	/* Remove the gconf volume notify*/
-	gconf_client_notify_remove (client, volume_notify_id);
+	gconf_client_notify_remove (gcd->client, volume_notify_id);
 	g_object_unref (gcd->cdrom);
 	bonobo_main_quit ();
 }
@@ -414,7 +414,7 @@ make_popup_menu (GnomeCD *gcd, GdkEventButton *event, gboolean iconify)
 		item = gtk_image_menu_item_new_with_mnemonic (_(menuitems[i].name));
 	        
 		/* If status is play, display the pause icon else display the play icon */
-        	if (status->audio == GNOME_CDROM_AUDIO_PLAY && i == 0) 
+        	if (status && status->audio == GNOME_CDROM_AUDIO_PLAY && i == 0) 
 	        	icon_name = GTK_STOCK_MEDIA_PAUSE;
 		else 
 		        icon_name = menuitems[i].icon;
@@ -442,7 +442,7 @@ make_popup_menu (GnomeCD *gcd, GdkEventButton *event, gboolean iconify)
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(item), image);
 		}
 
-		if (status->cd == GNOME_CDROM_STATUS_OK) {
+		if (status && status->cd == GNOME_CDROM_STATUS_OK) {
 			/* Previous menuitem is sensitive when it is
 			 * not the first track and we're playing */
 	        	if (i == 2) {
@@ -467,7 +467,7 @@ make_popup_menu (GnomeCD *gcd, GdkEventButton *event, gboolean iconify)
 		/* Disable the first four menu items when no disc is inserted
 		 * Update as per the buttons on the main window
 		 */
-		if ((status->cd == GNOME_CDROM_STATUS_NO_DISC || status->cd == GNOME_CDROM_STATUS_EMPTY_DISC) && i < 4) 
+		if ((status == NULL || status->cd == GNOME_CDROM_STATUS_NO_DISC || status->cd == GNOME_CDROM_STATUS_EMPTY_DISC) && i < 4) 
 			gtk_widget_set_sensitive (item, FALSE);
         
 		gtk_widget_show (item);
@@ -509,15 +509,15 @@ set_volume (GnomeCD *gcd)
 	gint scaled_volume;
 	double volume;
 	
-	scaled_volume = gconf_client_get_int (client, "/apps/gnome-cd/volume", NULL);
+	scaled_volume = gconf_client_get_int (gcd->client, "/apps/gnome-cd/volume", NULL);
 	/* On error set the Key */	
 	if (scaled_volume < 0) {
 		scaled_volume = 0;
-		gconf_client_set_int (client, "/apps/gnome-cd/volume", 0, NULL);
+		gconf_client_set_int (gcd->client, "/apps/gnome-cd/volume", 0, NULL);
 	}	
 	else if (scaled_volume > 100) {
 		scaled_volume = 100;
-		gconf_client_set_int (client, "/apps/gnome-cd/volume", 100, NULL);
+		gconf_client_set_int (gcd->client, "/apps/gnome-cd/volume", 100, NULL);
 	}
 	volume = ((double)scaled_volume/100)*255;
         
@@ -600,6 +600,7 @@ init_player (const char *device_override)
 	gcd = g_new0 (GnomeCD, 1);
 
 	gcd->not_ready = TRUE;
+	gcd->client = gconf_client_get_default ();
 	gcd->preferences = (GnomeCDPreferences *)preferences_new (gcd);
 	gcd->device_override = g_strdup (device_override);
 	
@@ -711,13 +712,17 @@ init_player (const char *device_override)
 
 	gcd->display = GTK_WIDGET (cd_display_new ());
 	GTK_WIDGET_SET_FLAGS (gcd->display, GTK_CAN_FOCUS | GTK_CAN_DEFAULT);
+	
+	gtk_widget_add_events (gcd->display, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK); 
+	
 	g_signal_connect (G_OBJECT (gcd->display), "loopmode-changed",
 			  G_CALLBACK (loopmode_changed_cb), gcd);
 	g_signal_connect (G_OBJECT (gcd->display), "playmode-changed",
 			  G_CALLBACK (playmode_changed_cb), gcd);
 	g_signal_connect (G_OBJECT (gcd->display), "style_set",
 			  G_CALLBACK (cd_display_set_style), NULL);
-
+	g_signal_connect (G_OBJECT (gcd->display), "remainingtime-mode-changed",
+			  G_CALLBACK (remainingtime_mode_changed_cb), gcd);
 	/* Theme needs to be loaded after the display is created */
 	gcd->theme = (GCDTheme *)theme_load (gcd, gcd->preferences->theme_name);
 	if (gcd->theme == NULL) {
@@ -791,7 +796,7 @@ init_player (const char *device_override)
 	
 	/* Set the initial volume */
 	set_volume (gcd);
-	volume_notify_id = gconf_client_notify_add (client,
+	volume_notify_id = gconf_client_notify_add (gcd->client,
 			"/apps/gnome-cd/volume", volume_changed_cb,
 			gcd, NULL, &error);
 	if (error != NULL) {
@@ -865,6 +870,7 @@ void
 tray_icon_create (GnomeCD *gcd)
 {
 	GdkPixbuf *pixbuf;
+	GdkPixbuf *icon;
 	GtkWidget *box;
 
 	/* Tray icon */
@@ -881,11 +887,17 @@ tray_icon_create (GnomeCD *gcd)
 				(GDestroyNotify) tray_object_destroyed);
 
 	gtk_container_add (GTK_CONTAINER (gcd->tray), box);
-	pixbuf = gdk_pixbuf_scale_simple (pixbuf_from_file ("gnome-cd/cd.png"),
+	
+	icon = pixbuf_from_file ("gnome-cd/cd.png");
+	pixbuf = gdk_pixbuf_scale_simple (icon,
 				16, 16, GDK_INTERP_BILINEAR);
 	gcd->tray_icon = gtk_image_new_from_pixbuf (pixbuf);
 	g_signal_connect (G_OBJECT (gcd->tray_icon), "expose_event",
 			 	G_CALLBACK (tray_icon_expose), gcd);
+			 	
+	g_object_unref (G_OBJECT (pixbuf));
+	g_object_unref (G_OBJECT (icon));
+
 	GTK_WIDGET_SET_FLAGS (box, GTK_CAN_FOCUS);
 	atk_object_set_name (gtk_widget_get_accessible (box), _("CD Player"));
 	gtk_container_add (GTK_CONTAINER (box), gcd->tray_icon);
@@ -965,23 +977,22 @@ register_stock_icons (void)
 int 
 main (int argc, char *argv[])
 {
-	static struct poptOption cd_popt_options [] = {
-#ifdef HAVE_GST
-		{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL },
-#endif
-		{ "device", '\0', POPT_ARG_STRING, &cd_option_device, 0,
+	static const GOptionEntry cd_goptions[] = {
+		{ "device", '\0', 0, G_OPTION_ARG_FILENAME, &cd_option_device,
 		  N_("CD device to use"), NULL },
-		{ "unique", '\0', POPT_ARG_NONE, &cd_option_unique, 0,
+		{ "unique", '\0', 0, G_OPTION_ARG_NONE, &cd_option_unique,
 		  N_("Only start if there isn't already a CD player application running"), NULL },
-		{ "play",   '\0', POPT_ARG_NONE, &cd_option_play, 0,
+		{ "play",   '\0', 0, G_OPTION_ARG_NONE, &cd_option_play,
 		  N_("Play the CD on startup"), NULL },
-		{ "tray", '\0', POPT_ARG_NONE, &cd_option_tray, 0,
+		{ "tray", '\0', 0, G_OPTION_ARG_NONE, &cd_option_tray,
 		  N_("Start iconified to tray"), NULL},
-		{ NULL, '\0', 0, NULL, 0 },
+		{ NULL, '\0', 0, 0, NULL, NULL, NULL }
 	};
 
 	GnomeCD *gcd;
 	GnomeClient *client;
+	GOptionGroup *group;
+	GOptionContext *ctx;
 
 	free (malloc (8)); /* -lefence */
 
@@ -993,31 +1004,19 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	ctx = g_option_context_new ("gnome-cd");
+
 #ifdef HAVE_GST
-	cd_popt_options[0].arg = (void*) gst_init_get_popt_table();
+	g_option_context_add_group (ctx, gst_init_get_option_group ());
 #endif
+
+	g_option_context_add_main_entries (ctx, cd_goptions, GETTEXT_PACKAGE);
+
 	gnome_program_init ("gnome-cd", VERSION, LIBGNOMEUI_MODULE, 
 			    argc, argv, 
-			    GNOME_PARAM_POPT_TABLE, cd_popt_options,
+			    GNOME_PARAM_GOPTION_CONTEXT, ctx,
 			    GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
-#ifdef HAVE_GST
-	if (!gst_scheduler_factory_get_default_name ()) {
-		GtkWidget *dialog;
-
-		dialog = gtk_message_dialog_new (NULL,
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Registry is not present or it is corrupted, please update it by running gst-register"));
-
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-
-		exit (1);
-	}
-#endif
-
 	register_stock_icons ();
 	client = gnome_master_client ();
 	g_signal_connect (client, "save_yourself",
